# aiomatter/events.py

from enum import StrEnum


class EventType(StrEnum):
    # ADDED_TO_TEAM = "added_to_team"
    # AUTHENTICATION_CHALLENGE = "authentication_challenge"
    # CHANNEL_CONVERTED = "channel_converted"
    # CHANNEL_CREATED = "channel_created"
    # CHANNEL_DELETED = "channel_deleted"
    # CHANNEL_MEMBER_UPDATED = "channel_member_updated"
    # CHANNEL_UPDATED = "channel_updated"
    # CHANNEL_VIEWED = "channel_viewed"
    # CONFIG_CHANGED = "config_changed"
    # DELETE_TEAM = "delete_team"
    # DIRECT_ADDED = "direct_added"
    # EMOJI_ADDED = "emoji_added"
    # EPHEMERAL_MESSAGE = "ephemeral_message"
    # GROUP_ADDED = "group_added"
    # HELLO = "hello"
    # LEAVE_TEAM = "leave_team"
    # LICENSE_CHANGED = "license_changed"
    # MEMBERROLE_UPDATED = "memberrole_updated"
    # NEW_USER = "new_user"
    # PLUGIN_DISABLED = "plugin_disabled"
    # PLUGIN_ENABLED = "plugin_enabled"
    # PLUGIN_STATUSES_CHANGED = "plugin_statuses_changed"
    POST_DELETED = "post_deleted"
    POST_EDITED = "post_edited"
    # POST_UNREAD = "post_unread"
    POSTED = "posted"
    # PREFERENCE_CHANGED = "preference_changed"
    # PREFERENCES_CHANGED = "preferences_changed"
    # PREFERENCES_DELETED = "preferences_deleted"
    # REACTION_ADDED = "reaction_added"
    # REACTION_REMOVED = "reaction_removed"
    # RESPONSE = "response"
    # ROLE_UPDATED = "role_updated"
    # STATUS_CHANGE = "status_change"
    # TYPING = "typing"
    # UPDATE_TEAM = "update_team"
    # USER_ADDED = "user_added"
    # USER_REMOVED = "user_removed"
    ANY = "*"
